@@ -1,4 +1,4 @@
-﻿/*
+/*
  *  Copyright (C) 2016 Enrico Mariotti <enricomariotti@yahoo.it>
  *  Copyright (C) 2017 KeePassXC Team <team@keepassxc.org>
  *
@@ -19,38 +19,69 @@
 #include "CsvImportWidget.h"
 #include "ui_CsvImportWidget.h"
 
+#include "core/Clock.h"
+#include "core/Database.h"
+#include "core/Group.h"
+#include "core/Totp.h"
+#include "format/CsvParser.h"
+#include "format/KeePass2Writer.h"
+#include "gui/csvImport/CsvParserModel.h"
+
 #include <QStringListModel>
 
-#include "core/Clock.h"
-#include "format/KeePass2Writer.h"
-#include "gui/MessageBox.h"
-#include "totp/totp.h"
+namespace
+{
+    // Extract group names from nested path and return the last group created
+    Group* createGroupStructure(Database* db, const QString& groupPath)
+    {
+        auto group = db->rootGroup();
+        if (!group || groupPath.isEmpty()) {
+            return group;
+        }
 
-// I wanted to make the CSV import GUI future-proof, so if one day you need a new field,
-// all you have to do is add a field to m_columnHeader, and the GUI will follow:
-// dynamic generation of comboBoxes, labels, placement and so on. Try it for immense fun!
+        auto nameList = groupPath.split("/", QString::SkipEmptyParts);
+        // Skip over first group name if root
+        if (nameList.first().compare("root", Qt::CaseInsensitive)) {
+            nameList.removeFirst();
+        }
+
+        for (const auto& name : qAsConst(nameList)) {
+            auto child = group->findChildByName(name);
+            if (!child) {
+                auto newGroup = new Group();
+                newGroup->setUuid(QUuid::createUuid());
+                newGroup->setName(name);
+                newGroup->setParent(group);
+                group = newGroup;
+            } else {
+                group = child;
+            }
+        }
+        return group;
+    }
+} // namespace
 
 CsvImportWidget::CsvImportWidget(QWidget* parent)
     : QWidget(parent)
     , m_ui(new Ui::CsvImportWidget())
     , m_parserModel(new CsvParserModel(this))
     , m_comboModel(new QStringListModel(this))
-    , m_columnHeader(QStringList() << QObject::tr("Group") << QObject::tr("Title") << QObject::tr("Username")
-                                   << QObject::tr("Password") << QObject::tr("URL") << QObject::tr("Notes")
-                                   << QObject::tr("TOTP") << QObject::tr("Icon") << QObject::tr("Last Modified")
-                                   << QObject::tr("Created"))
-    , m_fieldSeparatorList(QStringList() << ","
-                                         << ";"
-                                         << "-"
-                                         << ":"
-                                         << "."
-                                         << "\t")
 {
     m_ui->setupUi(this);
 
     m_ui->tableViewFields->setSelectionMode(QAbstractItemView::NoSelection);
     m_ui->tableViewFields->setFocusPolicy(Qt::NoFocus);
-    m_ui->messageWidget->setHidden(true);
+
+    m_columnHeader << QObject::tr("Group") << QObject::tr("Title") << QObject::tr("Username") << QObject::tr("Password")
+                   << QObject::tr("URL") << QObject::tr("Notes") << QObject::tr("TOTP") << QObject::tr("Icon")
+                   << QObject::tr("Last Modified") << QObject::tr("Created");
+
+    m_fieldSeparatorList << ","
+                         << ";"
+                         << "-"
+                         << ":"
+                         << "."
+                         << "\t";
 
     m_combos << m_ui->groupCombo << m_ui->titleCombo << m_ui->usernameCombo << m_ui->passwordCombo << m_ui->urlCombo
              << m_ui->notesCombo << m_ui->totpCombo << m_ui->iconCombo << m_ui->lastModifiedCombo << m_ui->createdCombo;
@@ -70,15 +101,12 @@ CsvImportWidget::CsvImportWidget(QWidget* parent)
     connect(m_ui->comboBoxFieldSeparator, SIGNAL(currentIndexChanged(int)), SLOT(parse()));
     connect(m_ui->checkBoxBackslash, SIGNAL(toggled(bool)), SLOT(parse()));
     connect(m_ui->checkBoxFieldNames, SIGNAL(toggled(bool)), SLOT(updatePreview()));
-
-    connect(m_ui->buttonBox, SIGNAL(accepted()), this, SLOT(writeDatabase()));
-    connect(m_ui->buttonBox, SIGNAL(rejected()), this, SLOT(reject()));
 }
 
 void CsvImportWidget::comboChanged(int index)
 {
     // this line is the one that actually updates GUI table
-    m_parserModel->mapColumns(index, m_combos.indexOf(qobject_cast<QComboBox*>(sender())));
+    m_parserModel->mapColumns(index - 1, m_combos.indexOf(qobject_cast<QComboBox*>(sender())));
     updateTableview();
 }
 
@@ -94,68 +122,81 @@ CsvImportWidget::~CsvImportWidget()
 
 void CsvImportWidget::configParser()
 {
-    m_parserModel->setBackslashSyntax(m_ui->checkBoxBackslash->isChecked());
-    m_parserModel->setComment(m_ui->comboBoxComment->currentText().at(0));
-    m_parserModel->setTextQualifier(m_ui->comboBoxTextQualifier->currentText().at(0));
-    m_parserModel->setCodec(m_ui->comboBoxCodec->currentText());
-    m_parserModel->setFieldSeparator(m_fieldSeparatorList.at(m_ui->comboBoxFieldSeparator->currentIndex()).at(0));
+    auto parser = m_parserModel->parser();
+    parser->setBackslashSyntax(m_ui->checkBoxBackslash->isChecked());
+    parser->setComment(m_ui->comboBoxComment->currentText().at(0));
+    parser->setTextQualifier(m_ui->comboBoxTextQualifier->currentText().at(0));
+    parser->setCodec(m_ui->comboBoxCodec->currentText());
+    parser->setFieldSeparator(m_fieldSeparatorList.at(m_ui->comboBoxFieldSeparator->currentIndex()).at(0));
 }
 
 void CsvImportWidget::updateTableview()
 {
-    m_ui->tableViewFields->resizeRowsToContents();
-    m_ui->tableViewFields->resizeColumnsToContents();
+    if (!m_buildingPreview) {
+        m_ui->tableViewFields->resizeRowsToContents();
+        m_ui->tableViewFields->resizeColumnsToContents();
 
-    for (int c = 0; c < m_ui->tableViewFields->horizontalHeader()->count(); ++c) {
-        m_ui->tableViewFields->horizontalHeader()->setSectionResizeMode(c, QHeaderView::Stretch);
+        for (int c = 0; c < m_ui->tableViewFields->horizontalHeader()->count(); ++c) {
+            m_ui->tableViewFields->horizontalHeader()->setSectionResizeMode(c, QHeaderView::Stretch);
+        }
     }
 }
 
 void CsvImportWidget::updatePreview()
 {
-    int minSkip = 0;
-    if (m_ui->checkBoxFieldNames->isChecked()) {
-        minSkip = 1;
-    }
+    m_buildingPreview = true;
+
+    int minSkip = m_ui->checkBoxFieldNames->isChecked() ? 1 : 0;
     m_ui->labelSizeRowsCols->setText(m_parserModel->getFileInfo());
     m_ui->spinBoxSkip->setRange(minSkip, qMax(minSkip, m_parserModel->rowCount() - 1));
     m_ui->spinBoxSkip->setValue(minSkip);
 
-    QStringList list(tr("Not Present"));
-    for (int i = 1; i < m_parserModel->getCsvCols(); ++i) {
+    QStringList csvColumns(tr("Not Present"));
+    auto parser = m_parserModel->parser();
+    for (int i = 0; i < parser->getCsvCols(); ++i) {
         if (m_ui->checkBoxFieldNames->isChecked()) {
-            auto columnName = m_parserModel->getCsvTable().at(0).at(i);
+            auto columnName = parser->getCsvTable().at(0).at(i);
             if (columnName.isEmpty()) {
-                list << QString(tr("Column %1").arg(i));
+                csvColumns << QString(tr("Column %1").arg(i));
             } else {
-                list << columnName;
+                csvColumns << columnName;
             }
         } else {
-            list << QString(tr("Column %1").arg(i));
+            csvColumns << QString(tr("Column %1").arg(i));
         }
     }
-    m_comboModel->setStringList(list);
+    m_comboModel->setStringList(csvColumns);
 
-    int j = 1;
-    for (QComboBox* b : m_combos) {
-        if (j < m_parserModel->getCsvCols()) {
-            b->setCurrentIndex(j);
-        } else {
-            b->setCurrentIndex(0);
+    // Try to match named columns to the combo boxes
+    for (int i = 0; i < m_columnHeader.size(); ++i) {
+        if (i >= m_combos.size()) {
+            // This should not happen, it is a programming error otherwise
+            Q_ASSERT(false);
+            break;
+        }
+
+        bool found = false;
+        for (int j = 0; j < csvColumns.size(); ++j) {
+            if (m_columnHeader.at(i).compare(csvColumns.at(j), Qt::CaseInsensitive) == 0) {
+                m_combos.at(i)->setCurrentIndex(j);
+                found = true;
+                break;
+            }
+        }
+        // Named column not found, default to "Not Present"
+        if (!found) {
+            m_combos.at(i)->setCurrentIndex(0);
         }
-        ++j;
     }
+
+    m_buildingPreview = false;
+    updateTableview();
 }
 
-void CsvImportWidget::load(const QString& filename, Database* const db)
+void CsvImportWidget::load(const QString& filename)
 {
-    // QApplication::processEvents();
-    m_db = db;
+    m_filename = filename;
     m_parserModel->setFilename(filename);
-    m_ui->labelFilename->setText(filename);
-    Group* group = m_db->rootGroup();
-    group->setUuid(QUuid::createUuid());
-    group->setNotes(tr("Imported from CSV file").append("\n").append(tr("Original data: ")) + filename);
     parse();
 }
 
@@ -163,42 +204,33 @@ void CsvImportWidget::parse()
 {
     configParser();
     QApplication::setOverrideCursor(Qt::WaitCursor);
-    // QApplication::processEvents();
+    QApplication::processEvents();
     bool good = m_parserModel->parse();
     updatePreview();
     QApplication::restoreOverrideCursor();
     if (!good) {
-        m_ui->messageWidget->showMessage(tr("Error(s) detected in CSV file!").append("\n").append(formatStatusText()),
-                                         MessageWidget::Warning);
-    } else {
-        m_ui->messageWidget->setHidden(true);
+        emit message(tr("Failed to parse CSV file: %1").arg(formatStatusText()));
     }
 }
 
-QString CsvImportWidget::formatStatusText() const
+QSharedPointer<Database> CsvImportWidget::buildDatabase()
 {
-    QString text = m_parserModel->getStatus();
-    int items = text.count('\n');
-    if (items > 2) {
-        return text.section('\n', 0, 1).append("\n").append(tr("[%n more message(s) skipped]", "", items - 2));
-    }
-    if (items == 1) {
-        text.append(QString("\n"));
-    }
-    return text;
-}
+    auto db = QSharedPointer<Database>::create();
+    db->rootGroup()->setNotes(tr("Imported from CSV file: %1").arg(m_filename));
 
-void CsvImportWidget::writeDatabase()
-{
-    setRootGroup();
     for (int r = 0; r < m_parserModel->rowCount(); ++r) {
         // use validity of second column as a GO/NOGO for all others fields
-        if (not m_parserModel->data(m_parserModel->index(r, 1)).isValid()) {
+        if (!m_parserModel->data(m_parserModel->index(r, 1)).isValid()) {
             continue;
         }
-        Entry* entry = new Entry();
+        auto group = createGroupStructure(db.data(), m_parserModel->data(m_parserModel->index(r, 0)).toString());
+        if (!group) {
+            continue;
+        }
+
+        auto entry = new Entry();
         entry->setUuid(QUuid::createUuid());
-        entry->setGroup(splitGroups(m_parserModel->data(m_parserModel->index(r, 0)).toString()));
+        entry->setGroup(group);
         entry->setTitle(m_parserModel->data(m_parserModel->index(r, 1)).toString());
         entry->setUsername(m_parserModel->data(m_parserModel->index(r, 2)).toString());
         entry->setPassword(m_parserModel->data(m_parserModel->index(r, 3)).toString());
@@ -208,7 +240,7 @@ void CsvImportWidget::writeDatabase()
         auto otpString = m_parserModel->data(m_parserModel->index(r, 6));
         if (otpString.isValid() && !otpString.toString().isEmpty()) {
             auto totp = Totp::parseSettings(otpString.toString());
-            if (totp->key.isEmpty()) {
+            if (!totp || totp->key.isEmpty()) {
                 // Bare secret, use default TOTP settings
                 totp = Totp::parseSettings({}, otpString.toString());
             }
@@ -257,99 +289,19 @@ void CsvImportWidget::writeDatabase()
         }
         entry->setTimeInfo(timeInfo);
     }
-    QBuffer buffer;
-    buffer.open(QBuffer::ReadWrite);
 
-    KeePass2Writer writer;
-    writer.writeDatabase(&buffer, m_db);
-    if (writer.hasError()) {
-        MessageBox::warning(this,
-                            tr("Error"),
-                            tr("CSV import: writer has errors:\n%1").arg(writer.errorString()),
-                            MessageBox::Ok,
-                            MessageBox::Ok);
-    }
-    emit editFinished(true);
+    return db;
 }
 
-void CsvImportWidget::setRootGroup()
+QString CsvImportWidget::formatStatusText() const
 {
-    QString groupLabel;
-    QStringList groupList;
-    bool is_root = false;
-    bool is_empty = false;
-    bool is_label = false;
-
-    for (int r = 0; r < m_parserModel->rowCount(); ++r) {
-        // use validity of second column as a GO/NOGO for all others fields
-        if (not m_parserModel->data(m_parserModel->index(r, 1)).isValid()) {
-            continue;
-        }
-        groupLabel = m_parserModel->data(m_parserModel->index(r, 0)).toString();
-        // check if group name is either "root", "" (empty) or some other label
-        groupList = groupLabel.split("/", QString::SkipEmptyParts);
-        if (groupList.isEmpty()) {
-            is_empty = true;
-        } else if (not groupList.first().compare("Root", Qt::CaseSensitive)) {
-            is_root = true;
-        } else if (not groupLabel.compare("")) {
-            is_empty = true;
-        } else {
-            is_label = true;
-        }
-
-        groupList.clear();
+    QString text = m_parserModel->parser()->getStatus();
+    int items = text.count('\n');
+    if (items > 2) {
+        return text.section('\n', 0, 1).append("\n").append(tr("[%n more message(s) skipped]", "", items - 2));
     }
-
-    if ((is_empty and is_root) or (is_label and not is_empty and is_root)) {
-        m_db->rootGroup()->setName("CSV IMPORTED");
-    } else {
-        m_db->rootGroup()->setName("Root");
+    if (items == 1) {
+        text.append(QString("\n"));
     }
-}
-
-Group* CsvImportWidget::splitGroups(const QString& label)
-{
-    // extract group names from nested path provided in "label"
-    Group* current = m_db->rootGroup();
-    if (label.isEmpty()) {
-        return current;
-    }
-
-    QStringList groupList = label.split("/", QString::SkipEmptyParts);
-    // avoid the creation of a subgroup with the same name as Root
-    if (m_db->rootGroup()->name() == "Root" && groupList.first() == "Root") {
-        groupList.removeFirst();
-    }
-
-    for (const QString& groupName : groupList) {
-        Group* children = hasChildren(current, groupName);
-        if (children == nullptr) {
-            Group* brandNew = new Group();
-            brandNew->setParent(current);
-            brandNew->setName(groupName);
-            brandNew->setUuid(QUuid::createUuid());
-            current = brandNew;
-        } else {
-            Q_ASSERT(children != nullptr);
-            current = children;
-        }
-    }
-    return current;
-}
-
-Group* CsvImportWidget::hasChildren(Group* current, const QString& groupName)
-{
-    // returns the group whose name is "groupName" and is child of "current" group
-    for (Group* group : current->children()) {
-        if (group->name() == groupName) {
-            return group;
-        }
-    }
-    return nullptr;
-}
-
-void CsvImportWidget::reject()
-{
-    emit editFinished(false);
+    return text;
 }
